package tareas.tarea1;

import java.awt.Point;

/**
 * Clase hija de Forma que implementa un Circulo.
 * @author Janther
 */
public class Circulo extends Forma {

    /**
     * Centro del Circulo.
     */
    private Point a = new Point();
    /**
     * Radio del Circulo.
     */
    private double r;

    /** Constructor por omision de la clase. Genera un Circulo de radio 0 y
     * centro en (0, 0).
     */
    public Circulo() {
        new Circulo(new Point(0, 0), 0);
    }

    /** Constructor de la clase. Genera un Circulo con su centro
     * en "p" y radio r.
     * @param p Punto donde se posicionara el centro del Circulo.
     * @param d Largo del radio.
     */
    public Circulo(final Point p, final double d) {
        a = p;
        r = d;
    }

    /** Escala la forma respecto al punto central en un factor dado.
     * @param centro Punto respecto del cual se hace el escalamiento.
     * @param factor  Factor usado en el escalamiento. >1 corresponde
     * a amplificacion, <1 corresponde a reduccion.
     */
    public void escalar(Point centro, double factor) {
        /*
         * Se traslado el punto central del Circulo con respecto al punto
         * "centro" en funcion del factor.
         */
        double nx;
        double ny;
        nx = (a.getX() - centro.getX()) * factor + centro.getX();
        ny = (a.getY() - centro.getY()) * factor + centro.getY();
        a.setLocation(nx, ny);
        /*
         * Se escalo el radio en funcion del factor.
         */
        r = r * factor;
    }

    /** Rota la forma respecto al punto central en un angulo dado.
     * @param centro Punto respecto del cual se hace la rotacion.
     * @param deltaAngulo Angulo de rotacion en radianes.
     */
    public void rotar(Point centro, double deltaAngulo) {
        /*
         * Se uso la transformacion rotacional para un vector.
         * http://es.wikipedia.org/wiki/Rotaci%C3%B3n
         */
        double nx, ny;
        nx = (a.getX() - centro.getX()) * Math.cos(deltaAngulo)
            - (a.getY() - centro.getY()) * Math.sin(deltaAngulo);
        ny = (a.getX() - centro.getX()) * Math.sin(deltaAngulo)
            + (a.getY() - centro.getY()) * Math.cos(deltaAngulo);
        a.setLocation(nx + centro.getX(), ny + centro.getY());
    }

    /** Traslada la forma desplazandola en sentido X e Y.
     * @param deltaX desplazamiento en sentido horizontal.
     * @param deltaY desplazamiento en sentido vertical.
     */
    public void trasladar(double deltaX, double deltaY) {
        /*
         * Se translado cada coordenada en su delta respectivo.
         */
        a.setLocation(a.getX() + deltaX, a.getY() + deltaY);
    }

    /** Redefine el metodo toString.
     * @return Los datos de este Objeto en forma de String.
     */
    public String toString() {
        String salida = "Circulo: centro = (";
        salida = salida + a.getX() + ", " + a.getY() + ")";
        salida = salida + ", radio = " + r + "\n";
        return salida;
    }

    /** Redefine el metodo clone.
     * @return Un Objeto clon de este.
     */
    public Object clone() {
        return new Circulo((Point) a.clone(), r);
    }

    /** Redefine el metodo equals.
     * @param cir Circulo a comparar.
     * @return Verdadero si los Objetos son iguales.
     */
    public boolean equals(Object cir) {
        /*
         * Se ve si coinciden las clases y luego se ve que los campos sean
         * equivalentes.
         */
        if (cir instanceof Circulo) {
            if (a.equals(((Circulo) cir).getA())) {
                if (r == ((Circulo) cir).getR()) {
                    return true;
                }
            }
        }
        return false;
    }

    /** Retorna el centro del Circulo.
     * @return El centro del Circulo.
     */
    public Point getA() {
        return a;
    }

    /** Retorna el radio del Circulo.
     * @return El radio del Circulo
     */
    public double getR() {
        return r;
    }
}
