package tareas.tarea1;

import java.awt.Point;
import java.util.ArrayList;

/**
 * Clase hija de Forma que implementa un conjunto de Formas.
 * @author Janther
 */
public class Figura extends Forma {

    /**
     * Arreglo de las formas que seran almacenadas.
     */
    public ArrayList < Forma > formas = new ArrayList < Forma >();

    /** Constructor por omision de la clase.
     */
    public Figura() {
    }

    /** Escala la forma respecto al punto central en un factor dado.
     * @param centro Punto respecto del cual se hace el escalamiento.
     * @param factor  Factor usado en el escalamiento. >1 corresponde
     * a amplificacion, <1 corresponde a reduccion.
     */
    public void escalar(Point centro, double factor) {
        /*
         * Por cada Forma en Figura se ejecuta el metodo escalar.
         */
        for (int i = 0; i < formas.size(); i++) {
            formas.get(i).escalar(centro, factor);
        }
    }

    /** Rota la forma respecto al punto central en un angulo dado.
     * @param centro Punto respecto del cual se hace la rotacion.
     * @param deltaAngulo angulo de rotacion en radianes.
     */
    public void rotar(Point centro, double deltaAngulo) {
        /*
         * Por cada Forma en Figura se ejecuta el metodo rotar(Point, double).
         */
        for (int i = 0; i < formas.size(); i++) {
            formas.get(i).rotar(centro, deltaAngulo);
        }
    }

    /** Traslada la forma desplazandola en sentido X e Y.
     * @param deltaX desplazamiento en sentido horizontal.
     * @param deltaY desplazamiento en sentido vertical.
     */
    public void trasladar(double deltaX, double deltaY) {
        /*
         * Por cada Forma en Figura se ejecuta el metodo trasladar.
         */
        for (int i = 0; i < formas.size(); i++) {
            formas.get(i).trasladar(deltaX, deltaY);
        }
    }

    /** Redefine el metodo toString.
     * @return Los datos de este Objeto en forma de String.
     */
    public String toString() {
        /*
         * Por cada Forma en Figura se ejecuta el metodo toString.
         */
        String salida;
        if (formas.size() > 0) {
            salida = "Esta Figura contiene las siguientes Formas:\n";
            for (int i = 0; i < formas.size(); i++) {
                salida = salida + formas.get(i).toString();
            }
        } else {
            return "Esta figura no contiene Formas.";
        }
        return salida;
    }

    /** Agrega una forma a la figura.
     * @param f Forma a agregar.
     */
    public void addForma(Forma f) {
        formas.add(f);
    }

    /** Remueve una forma de la figura.
     * @param f Forma a remover.
     */
    public void removeForma(Forma f) {
        formas.remove(f);
    }

    /** Retorna una forma.
     * @param i ID de la forma.
     * @return La forma con el ID i;
     */
    public Forma getForma(int i) {
        return ((Forma) formas.get(i));
    }
}
