package tareas.tarea1;

import java.awt.Point;
import java.util.ArrayList;

/**
 * Clase hija de Forma que implementa un Rectangulo.
 * @author Janther
 */
public class Rectangulo extends Forma {

    /**
     * Numero de vertices en la Forma.
     */
    private final int numVertices = 4;
    /**
     * Vertices del Rectangulo.
     */
    private Point[] vertices = new Point[numVertices];

    /**
     * Constructor por omisiun de la clase. Genera un Rectangulo con
     * sus vertices en (0,0).
     */
    public Rectangulo() {
        this(new Point(), new Point());
    }

    /** Constructor de la clase. Genera un Rectangulo con sus vertices
     * opuestos en los puntos dados y ejes paralelos a los ejes coordenados.
     * @param p1 Punto donde se posicionara un vertice del Rectangulo.
     * @param p2 Punto donde se posicionara un vertice opuesto del Rectangulo.
     */
    public Rectangulo(final Point p1, final Point p2) {
        double x1 = p1.getX();
        double y1 = p1.getY();
        double x2 = p2.getX();
        double y2 = p2.getY();

        for (int i = 0; i < numVertices; i++) {
            vertices[i] = new Point();
        }

        vertices[0].setLocation(x1, y1);
        vertices[1].setLocation(x2, y1);
        vertices[2].setLocation(x1, y2);
        vertices[3].setLocation(x2, y2);
    }

    /** Constructor de la clase. Genera un Rectangulo con sus vertices
     * en los puntos dados.
     * @param puntos Vertices del Rectangulo.
     */
    public Rectangulo(final Point[] puntos) {
        for (int i = 0; i < numVertices; i++) {
            vertices[i] = new Point();
            vertices[i].setLocation(puntos[i]);
        }
    }

    /** Escala la forma respecto al punto central en un factor dado.
     * @param centro Punto respecto del cual se hace el escalamiento.
     * @param factor  Factor usado en el escalamiento. >1 corresponde
     * a amplificacion, <1 corresponde a reduccion.
     */
    public void escalar(final Point centro, double factor) {
        /*
         * Se traslada cada punto del Rectangulo con respecto al punto "centro"
         * en funcion del factor.
         */
        double nx, ny;
        for (int i = 0; i < numVertices; i++) {
            nx = (vertices[i].getX() - centro.getX()) * factor + centro.getX();
            ny = (vertices[i].getY() - centro.getY()) * factor + centro.getY();
            vertices[i].setLocation(nx, ny);
        }
    }

    /** Rota la forma respecto al punto central en un angulo dado.
     * @param centro Punto respecto del cual se hace la rotacion.
     * @param deltaAngulo angulo de rotacion en radianes.
     */
    public void rotar(Point centro, double deltaAngulo) {
        /*
         * Se uso la transformacion rotacional para un vector en cada punto.
         * http://es.wikipedia.org/wiki/Rotaci%C3%B3n
         */
        double nx, ny;
        for (int i = 0; i < numVertices; i++) {
            nx = (vertices[i].getX() - centro.getX()) * Math.cos(deltaAngulo)
                - (vertices[i].getY() - centro.getY()) * Math.sin(deltaAngulo);
            ny = (vertices[i].getX() - centro.getX()) * Math.sin(deltaAngulo)
                + (vertices[i].getY() - centro.getY()) * Math.cos(deltaAngulo);
            vertices[i].setLocation(nx + centro.getX(), ny + centro.getY());
        }
    }

    /** Traslada la forma desplazandola en sentido X e Y.
     * @param deltaX desplazamiento en sentido horizontal.
     * @param deltaY desplazamiento en sentido vertical.
     */
    public void trasladar(double deltaX, double deltaY) {
        /*
         * Se translado cada coordenada en su delta respectivo.
         */
        for (int i = 0; i < numVertices; i++) {
            vertices[i].setLocation(vertices[i].getX() + deltaX,
                                    vertices[i].getY() + deltaY);
        }
    }

    /** Redefine el metodo toString.
     * @return Los datos de este Objeto en forma de String.
     */
    public String toString() {
        String salida = "Rectangulo: vertices  = {(";
        salida = salida + vertices[0].getX() + ", " + vertices[1].getY() + ")";
        for (int i = 1; i < numVertices; i++) {
            salida = salida + ", (" + vertices[i].getX()
                     + ", " + vertices[i].getY() + ")";
        }
        salida = salida + "}\n";
        return salida;
    }

    /** Redefine el metodo clone.
     * @return Un Objeto clon de este.
     */
    public Object clone() {
        return new Rectangulo(vertices);
    }

    /** Redefine el metodo equals.
     * @param r Rectangulo a comparar.
     * @return Verdadero si los Objetos son iguales.
     */
    public boolean equals(Object r) {
        /*
         * Se ve si coinciden las clases.
         */
        if (r instanceof Rectangulo) {
            /*
             * Se usa ArrayList pues se nececita el m�todo remove sin que
             * afecte a los puntos.
             */
            ArrayList < Point > puntos = new ArrayList < Point >();
            for (int i = 0; i < ((Rectangulo) r).getVertices().length; i++) {
                puntos.add(((Rectangulo) r).getVertices()[i]);
            }

            for (int i = 0; i < numVertices; i++) {
                for (int j = 0; j < puntos.size(); j++) {
                    if (vertices[i].equals(puntos.get(j))) {
                        /*
                         * Si un punto es equivalente a otro se descarta del
                         * ArrayList.
                         * Una vez que se ha comprobado que todos los puntos
                         * tienen su equivalente, los rect�ngulos son iguales.
                         */
                        puntos.remove(j);
                        if (puntos.size() == 0) {
                            return true;
                        }
                        /*
                         * Si se encontr� un equivalente se debe proceder con
                         * el siguiente punto.
                         */
                        break;
                    }
                }
            }
        }
        return false;
    }

    /** Retorna los vertices del Rectangulo.
     * @return Vertices del Rectangulo.
     */
    public Point[] getVertices() {
        return vertices;
    }
}
