package tareas.tarea2.electronica;

import java.awt.Point;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.geom.Line2D;
import java.awt.geom.Rectangle2D;

/**
 * Clase hija de Forma que implementa un punto de tierra.
 * @author Janther
 */
public class Cable extends Componente {

    /**
     * Numero de nodos en el Componente.
     */
    private final int numNodos = 2;
    /**
     * Nodos del Componente.
     */
    private Nodo[] nodos = new Nodo[numNodos];
    /**
     * Linea que conforman el cable y el area seleccionable del componente.
     */
    private Line2D linea;
    /**
     * Color de la forma.
     */
    private Color c;

    /**
     * Constructor por omision del componente, lo genera en la esquina superior
     * izquierda del panel.
     * @param a Es uno de los nodos ubicados a un extremo del cable.
     * @param b Es uno de los nodos ubicados a un extremo del cable.
     */
    public Cable(Nodo a, Nodo b) {
        nodos[0] = a;
        nodos[1] = b;
        a.addCable(this);
        b.addCable(this);
        setArea();
    }

    /**
     * Setea el area seleccionable del componente en base a los puntos que lo
     * conforman.
     */
    public void setArea() {
        linea = new Line2D.Double(nodos[0].getPunto(), nodos[1].getPunto());
    }

    /** Escala la forma respecto al punto central en un factor dado.
     * @param centro Punto respecto del cual se hace el escalamiento.
     * @param factor  Factor usado en el escalamiento. >1 corresponde
     * a amplificacion, <1 corresponde a reduccion.
     */
    public void escalar(Point centro, double factor) { }

    /** Rota la forma respecto al punto central en un angulo dado.
     * @param centro Punto respecto del cual se hace la rotacion.
     * @param deltaAngulo angulo de rotacion en radianes.
     */
    public void rotar(Point centro, double deltaAngulo) { }

    /** Rota la forma respecto a su centro en 90 grados.
     */
    public void rotar() { }

    /** Traslada la forma desplazandola en sentido X e Y.
     * @param deltaX desplazamiento en sentido horizontal.
     * @param deltaY desplazamiento en sentido vertical.
     */
    public void trasladar(double deltaX, double deltaY) { }

    /**
     * Verifica si un punto esta dentro de el area seleccionable de la forma.
     * @param p Punto a verificar.
     * @return Verdadero si el punto esta dentro del area.
     */
    public boolean isSelected(Point p) {
        if (linea.intersects(new Rectangle2D.Double(
                p.getX(), p.getY(), 1, 1))) {
            c = Color.RED;
            return true;
        }
        c = Color.BLACK;
        return false;
    }

    /** Dibuja la Forma en el Graphics2D.
     * @param g2d Grafico en el cual se incluira el dibujo.
     */
    public void paintComponent(Graphics2D g2d) {
        g2d.setColor(c);
        linea = new Line2D.Double(nodos[0].getPunto(), nodos[1].getPunto());
        g2d.draw(linea);
    }

    /** Retorna los nodos relacionados a este Componente.
     * @return Arreglo de nodos.
     */
    public Nodo[] getNodos() {
        return new Nodo[0];
    }

    /** Retorna el punto que basta para localizar la coordenada de un
     * componente, no asi su orientacion.
     * @return Punto con las coordenadas del componente.
     */
    public Point getPunto() {
        return nodos[0].getPunto();
    }

    /** Redefine el metodo clone.
     * @return Un Objeto clon de este.
     */
    public Object clone() {
        return new Cable((Nodo) nodos[0].clone(), (Nodo) nodos[1].clone());
    }

    /**
     * Se remueve a si mismo de todos los nodos a los que esta relacionado.
     */
    public void destroy() {
        nodos[0].removeCable(this);
        nodos[1].removeCable(this);
    }
}
